#define CHARGE_PIN 3
#define DISCHARGE_PIN 2
#define VOLTAGE_CHECK_PIN 0
// Callibrating definitions
// Value for 0% humidity
#define HUMIDITY_0 4000
// Value for 100% humidity
#define HUMIDITY_100 4716

// 1 - Callibration mode
// 0 - Measurement in units mode
#define CALIBRATION 1

int counter = 0;


unsigned long charge()
{
  // Setting pins in charging mode
  pinMode(DISCHARGE_PIN, INPUT);
  unsigned long begin_time = micros(); // Loading the initial time
  pinMode(CHARGE_PIN, OUTPUT);
  digitalWrite(CHARGE_PIN, HIGH);
  // Waiting for the capacitor to reach 63,2% of supply voltage
  while (analogRead(VOLTAGE_CHECK_PIN) < 648);
  unsigned long end_time = micros(); // Loading the final time
  return end_time - begin_time; // Calculating the charging time
}


void discharge()
{
  // Setting pins in discharging mode
  pinMode(CHARGE_PIN, INPUT);
  digitalWrite(CHARGE_PIN, LOW);
  pinMode(DISCHARGE_PIN, OUTPUT);
  digitalWrite(DISCHARGE_PIN, LOW);
  // Waiting for the capacitor to reach 0V
  while (analogRead(VOLTAGE_CHECK_PIN) > 0);
}


unsigned int get_humidity()
{
  discharge();
  unsigned long charge_time = charge(); // Charging with time measurement
  // Skalowanie
  unsigned int humidity = map(charge_time, HUMIDITY_0, HUMIDITY_100, 0, 100);
  // Callibration mode (1) or measurement mode (0)
  if (CALIBRATION == 1)
    return charge_time;
  else
    return humidity;
}


void setup()
{
  Serial.begin(9600);
}


void loop()
{
  unsigned int humidity;
  // Sensor requires frequent measurement
  // Every hundredth measurement is displayed
  while (counter < 100)
  {
    humidity = get_humidity();
    counter++;
  }
  Serial.println(humidity, DEC);
  counter = 0;
}
